// ../packages/core/index.ts
import { createUnplugin } from "unplugin";
import { JSX_TSX_REG, NAME, SUPPORT_FILE_REG } from "../utils/index.js";
import { createFilter } from "@rollup/pluginutils";
import { parse as parse3 } from "@vue/compiler-sfc";
import chalk from "chalk";

// ../packages/core/runtime/pre-process-css.ts
import { parse as parse2, resolve as resolve2 } from "path";
import fg from "fast-glob";
import fs from "fs-extra";
import {
  FG_IGNORE_LIST,
  SUPPORT_FILE as SUPPORT_FILE2,
  completeSuffix as completeSuffix2,
  transformSymbol as transformSymbol2
} from "../utils/index.js";

// ../packages/core/parser/parser-variable.ts
import { parse as babelParse } from "@babel/parser";

// ../node_modules/.pnpm/estree-walker@3.0.3/node_modules/estree-walker/src/walker.js
var WalkerBase = class {
  constructor() {
    this.should_skip = false;
    this.should_remove = false;
    this.replacement = null;
    this.context = {
      skip: () => this.should_skip = true,
      remove: () => this.should_remove = true,
      replace: (node) => this.replacement = node
    };
  }
  /**
   * @template {Node} Parent
   * @param {Parent | null | undefined} parent
   * @param {keyof Parent | null | undefined} prop
   * @param {number | null | undefined} index
   * @param {Node} node
   */
  replace(parent, prop, index, node) {
    if (parent && prop) {
      if (index != null) {
        parent[prop][index] = node;
      } else {
        parent[prop] = node;
      }
    }
  }
  /**
   * @template {Node} Parent
   * @param {Parent | null | undefined} parent
   * @param {keyof Parent | null | undefined} prop
   * @param {number | null | undefined} index
   */
  remove(parent, prop, index) {
    if (parent && prop) {
      if (index !== null && index !== void 0) {
        parent[prop].splice(index, 1);
      } else {
        delete parent[prop];
      }
    }
  }
};

// ../node_modules/.pnpm/estree-walker@3.0.3/node_modules/estree-walker/src/sync.js
var SyncWalker = class extends WalkerBase {
  /**
   *
   * @param {SyncHandler} [enter]
   * @param {SyncHandler} [leave]
   */
  constructor(enter, leave) {
    super();
    this.should_skip = false;
    this.should_remove = false;
    this.replacement = null;
    this.context = {
      skip: () => this.should_skip = true,
      remove: () => this.should_remove = true,
      replace: (node) => this.replacement = node
    };
    this.enter = enter;
    this.leave = leave;
  }
  /**
   * @template {Node} Parent
   * @param {Node} node
   * @param {Parent | null} parent
   * @param {keyof Parent} [prop]
   * @param {number | null} [index]
   * @returns {Node | null}
   */
  visit(node, parent, prop, index) {
    if (node) {
      if (this.enter) {
        const _should_skip = this.should_skip;
        const _should_remove = this.should_remove;
        const _replacement = this.replacement;
        this.should_skip = false;
        this.should_remove = false;
        this.replacement = null;
        this.enter.call(this.context, node, parent, prop, index);
        if (this.replacement) {
          node = this.replacement;
          this.replace(parent, prop, index, node);
        }
        if (this.should_remove) {
          this.remove(parent, prop, index);
        }
        const skipped = this.should_skip;
        const removed = this.should_remove;
        this.should_skip = _should_skip;
        this.should_remove = _should_remove;
        this.replacement = _replacement;
        if (skipped)
          return node;
        if (removed)
          return null;
      }
      let key;
      for (key in node) {
        const value = node[key];
        if (value && typeof value === "object") {
          if (Array.isArray(value)) {
            const nodes = (
              /** @type {Array<unknown>} */
              value
            );
            for (let i = 0; i < nodes.length; i += 1) {
              const item = nodes[i];
              if (isNode(item)) {
                if (!this.visit(item, node, key, i)) {
                  i--;
                }
              }
            }
          } else if (isNode(value)) {
            this.visit(value, node, key, null);
          }
        }
      }
      if (this.leave) {
        const _replacement = this.replacement;
        const _should_remove = this.should_remove;
        this.replacement = null;
        this.should_remove = false;
        this.leave.call(this.context, node, parent, prop, index);
        if (this.replacement) {
          node = this.replacement;
          this.replace(parent, prop, index, node);
        }
        if (this.should_remove) {
          this.remove(parent, prop, index);
        }
        const removed = this.should_remove;
        this.replacement = _replacement;
        this.should_remove = _should_remove;
        if (removed)
          return null;
      }
    }
    return node;
  }
};
function isNode(value) {
  return value !== null && typeof value === "object" && "type" in value && typeof value.type === "string";
}

// ../node_modules/.pnpm/estree-walker@3.0.3/node_modules/estree-walker/src/index.js
function walk(ast, { enter, leave }) {
  const instance = new SyncWalker(enter, leave);
  return instance.visit(ast, null);
}

// ../packages/core/parser/parser-variable.ts
import { extend, isEmptyObj } from "../utils/index.js";
var getVariable = (descriptor) => {
  let variableName = {};
  variableName = getVariableNameBySetup(setScriptContent(descriptor, "setup"));
  variableName = getVariableNameByScript(setScriptContent(descriptor, "script"), variableName);
  return variableName;
};
function setScriptContent(descriptor, type) {
  let content = "";
  if (descriptor.scriptSetup && type === "setup")
    content = descriptor.scriptSetup.content;
  if (descriptor.script && type === "script")
    content = descriptor.script.content;
  return content;
}
function getVariableNameBySetup(content, contextAst) {
  const variableNameBySetup = {};
  if (!content && !contextAst)
    return variableNameBySetup;
  const ast = contextAst || babelParse(content, {
    sourceType: "module",
    plugins: ["typescript"]
  });
  walk(ast, {
    enter(node, parent) {
      if (parent && parent.type === "Program" && node.type === "VariableDeclaration") {
        const declarations = node.declarations;
        declarations.forEach((declare) => {
          const identifier = declare.id;
          variableNameBySetup[identifier.name] = declare.init;
        });
      }
    }
  });
  return variableNameBySetup;
}
function getVariableNameByScript(content, variableName) {
  if (!content)
    return variableName;
  let variableNameInScript = {};
  let isEmptyVariableNames = false;
  let setupIndex = 0;
  let dataIndex = 0;
  let hasSetup = false;
  let hasData = false;
  let setupReturnNode = {};
  let dataReturnNode = {};
  let index = 2;
  if (isEmptyObj(variableName))
    isEmptyVariableNames = true;
  const ast = babelParse(content, {
    sourceType: "module",
    plugins: ["typescript"]
  });
  walk(ast, {
    enter(node, parent) {
      if (parent && parent.type === "ObjectMethod" && node.type === "Identifier" && node.name === "setup") {
        hasSetup = true;
        setupIndex = index;
        index--;
      }
      if (parent && parent.type === "ObjectMethod" && node.type === "Identifier" && node.name === "data") {
        hasData = true;
        dataIndex = index;
        index--;
      }
      if (parent && parent.type === "ReturnStatement" && node.type === "ObjectExpression" && hasSetup) {
        hasSetup = false;
        setupReturnNode = node;
      }
      if (parent && parent.type === "ReturnStatement" && node.type === "ObjectExpression" && hasData) {
        hasData = false;
        dataReturnNode = node;
      }
    }
  });
  if (setupIndex === 0 && dataIndex === 0) {
    if (isEmptyVariableNames) {
      return variableName;
    } else {
      variableNameInScript = getVariableNameBySetup("", ast);
    }
    variableNameInScript = extend(variableNameInScript, variableName);
    setupReturnNode = {};
    dataReturnNode = {};
    return variableNameInScript;
  }
  const setupReturnRes = getObjectExpressionReturnNode(setupReturnNode);
  const dataReturnRes = getObjectExpressionReturnNode(dataReturnNode);
  variableNameInScript = setupIndex > dataIndex ? extend(setupReturnRes, dataReturnRes) : extend(dataReturnRes, setupReturnRes);
  if (!isEmptyVariableNames)
    variableNameInScript = extend(variableNameInScript, variableName);
  return variableNameInScript;
}
function getObjectExpressionReturnNode(node) {
  const res = {};
  if (!node.properties)
    return res;
  node.properties.forEach((value) => {
    const key = value.key;
    res[key.name] = value.value;
  });
  return res;
}
function matchVariable(importCSSModule, variableName) {
  const res = [];
  importCSSModule.forEach((val) => {
    const varNode = variableName[val];
    const resObj = {
      value: val,
      has: false,
      isRef: false
    };
    if (varNode) {
      resObj.has = true;
      if (varNode.type === "CallExpression" && varNode.callee.type === "Identifier" && varNode.callee.name === "ref")
        resObj.isRef = true;
    }
    res.push(resObj);
  });
  return res;
}

// ../packages/core/parser/parser-import.ts
var innerAtRule = "media,extend,at-root,debug,warn,forward,mixin,include,function,error,keyframes,font-face,page,supports,namespace,return,if,else,for,while,each,content,tailwind,apply,layer,screen,responsive,variants,config,applyRule";
function parseImports(content, helper) {
  const imports = [];
  let currentImport;
  let state = 0 /* Initial */;
  let i = 0;
  let AtPath = "";
  const source = content;
  while (i < source.length) {
    const char = source[i];
    if (/[\r\t\f\v\\]/g.test(char)) {
      i++;
      continue;
    }
    switch (state) {
      case 0 /* Initial */:
        if (char === "@")
          state = 3 /* AtStart */;
        if (char === "/" && source[i + 1] === "/")
          state = 1 /* InlineComment */;
        if (char === "/" && source[i + 1] === "*")
          state = 2 /* Comment */;
        break;
      case 1 /* InlineComment */:
        if (char === "\n")
          state = 0 /* Initial */;
        break;
      case 2 /* Comment */:
        if (char === "*" && source[i + 1] === "/")
          state = 0 /* Initial */;
        break;
      case 3 /* AtStart */:
        if (/[A-Za-z]$/.test(char))
          AtPath = AtPath + char;
        else
          state = 4 /* AtEnd */;
        if (i === source.length - 1) {
          if (char === '"' || char === "'")
            throw new Error("syntax error: unmatched quotes");
          else
            walkContentEnd(i);
        }
        if (!/[A-Za-z]$/.test(char) && char !== "\n" && char !== " " && char !== "-")
          throw new Error("syntax error");
        break;
      case 4 /* AtEnd */:
        if (char !== "\n" && char !== " " && char !== "-") {
          if (char === "/")
            throw new Error("syntax error");
          if (AtPath === "import") {
            AtPath = "";
            state = 5 /* AtImport */;
            currentImport = { type: "import", path: "" };
            i--;
          } else if (AtPath === "use") {
            AtPath = "";
            state = 6 /* AtUse */;
            currentImport = { type: "use", path: "" };
            i--;
          } else if (AtPath === "require") {
            AtPath = "";
            state = 7 /* AtRequire */;
            currentImport = { type: "require", path: "" };
            i--;
          } else {
            if (!innerAtRule.includes(AtPath))
              throw new Error("syntax error: unknown At Rule");
            AtPath = "";
            state = 0 /* Initial */;
          }
        }
        break;
      case 5 /* AtImport */:
      case 6 /* AtUse */:
      case 7 /* AtRequire */:
        if (char === "@" && !/[A-Za-z]$/.test(source[i - 1])) {
          i--;
          state = 0 /* Initial */;
          break;
        }
        if (char === "/" && (source[i + 1] === "/" || source[i + 1] === "*")) {
          i--;
          state = 0 /* Initial */;
          break;
        }
        if (char === "'" || char === '"') {
          currentImport.start = i;
          state = 8 /* QuotesStart */;
          break;
        }
        if (char !== "\n" && char !== " ") {
          currentImport.start = i;
          currentImport.path += char;
          state = 10 /* StringLiteral */;
          break;
        }
        break;
      case 8 /* QuotesStart */:
        if (char === "'" || char === '"') {
          currentImport.end = i;
          state = 9 /* QuotesEnd */;
          if (i === source.length - 1)
            walkContentEnd(i);
          break;
        }
        if (i === source.length - 1)
          throw new Error("syntax error: unmatched quotes");
        currentImport.path += char;
        break;
      case 9 /* QuotesEnd */:
        if (i === source.length - 1 || char === ";" || char === "\n") {
          walkContentEnd(i);
        } else {
          i--;
          state = 10 /* StringLiteral */;
        }
        break;
      case 10 /* StringLiteral */:
        if (char === ";" || char === "\n") {
          walkContentEnd(i);
          break;
        }
        if (char !== "@" && (char === " " || char === "," || char === '"' || char === "'")) {
          const curType = currentImport?.type;
          walkContentEnd(i);
          if (curType === "import") {
            state = 5 /* AtImport */;
            currentImport = { type: "import", path: "" };
          } else if (curType === "use") {
            state = 6 /* AtUse */;
            currentImport = { type: "use", path: "" };
          } else if (curType === "require") {
            state = 7 /* AtRequire */;
            currentImport = { type: "require", path: "" };
          }
          if (char === "'" || char === '"') {
            currentImport.start = i;
            state = 8 /* QuotesStart */;
            if (i === source.length - 1 && (char === '"' || char === "'"))
              throw new Error("syntax error: unmatched quotes");
            break;
          }
          break;
        }
        currentImport.path += char;
        if (i === source.length - 1)
          walkContentEnd(i);
        break;
    }
    i++;
  }
  function walkContentEnd(index) {
    pushCurrentImport(index);
    state = 0 /* Initial */;
  }
  function pushCurrentImport(index) {
    if (currentImport && currentImport.start !== void 0) {
      currentImport.end = index;
      if (helper) {
        helper.forEach((fn) => {
          currentImport = fn(currentImport);
        });
      }
      imports.push(currentImport);
      currentImport = void 0;
    }
  }
  function getCurState() {
    return state;
  }
  function getCurImport() {
    return currentImport;
  }
  return {
    imports,
    getCurState,
    getCurImport
  };
}

// ../packages/core/parser/parser-vbindm.ts
function parseVBindM(source) {
  const result = /* @__PURE__ */ new Set();
  let state = 0 /* Initial */;
  let i = 0;
  let curBuffer = "";
  let mark = 0;
  while (i < source.length) {
    const char = source[i];
    switch (state) {
      case 0 /* Initial */:
        if (char === "/" && source[i + 1] === "/")
          state = 1 /* InlineComment */;
        if (char === "/" && source[i + 1] === "*")
          state = 2 /* Comment */;
        if (char === "v")
          state = 3 /* VBindM */;
        if (char === ")")
          mark--;
        if (char === "(")
          mark++;
        break;
      case 1 /* InlineComment */:
        if (char === "\n")
          state = 0 /* Initial */;
        break;
      case 2 /* Comment */:
        if (char === "*" && source[i + 1] === "/")
          state = 0 /* Initial */;
        if (i === source.length - 1)
          throw new Error("syntax error: unmatched */");
        break;
      case 3 /* VBindM */:
        if (char === "-" && source[i + 1] === "b" && source[i + 2] === "i" && source[i + 3] === "n" && source[i + 4] === "d" && source[i + 5] === "-" && source[i + 6] === "m") {
          state = 4 /* VBindMValueStart */;
          i = i + 6;
        }
        break;
      case 4 /* VBindMValueStart */:
        if (char !== "(") {
          i--;
          state = 5 /* VBindMValue */;
          break;
        }
        mark++;
        break;
      case 5 /* VBindMValue */:
        if (char !== ")") {
          curBuffer = curBuffer + char;
        } else {
          i--;
          state = 6 /* VBindMValueEnd */;
        }
        break;
      case 6 /* VBindMValueEnd */:
        if (char === ")") {
          mark--;
          if (curBuffer) {
            result.add(curBuffer);
            curBuffer = "";
          }
          state = 0 /* Initial */;
        }
        break;
    }
    i++;
  }
  if (mark > 0)
    throw new Error("syntax error: unmatched )");
  if (mark < 0)
    throw new Error("syntax error: unmatched (");
  return [...result];
}

// ../packages/core/transform/transform-quotes.ts
function transformQuotes(importer2) {
  if (!importer2.path.startsWith('"') && !importer2.path.endsWith('"')) {
    if (importer2.path.startsWith("'") && importer2.path.endsWith("'"))
      importer2.path = `"${importer2.path.slice(1, -1)}"`;
    else
      importer2.path = `"${importer2.path}"`;
  }
  return importer2;
}

// ../packages/core/runtime/process-css.ts
import { parse, resolve } from "path";
import { SUPPORT_FILE, completeSuffix, setTArray, transformSymbol } from "../utils/index.js";
var getCSSFileRecursion = (lang = SUPPORT_FILE.CSS, key, cssFiles, cb, sfcPath, matchedMark = /* @__PURE__ */ new Set()) => {
  key = completeSuffix(key, lang);
  if (!cssFiles.get(key))
    key = completeSuffix(key, SUPPORT_FILE.CSS, true);
  if (matchedMark.has(key))
    return;
  const cssFile = cssFiles.get(key);
  if (cssFile) {
    if (!cssFile.sfcPath)
      cssFile.sfcPath = /* @__PURE__ */ new Set();
    cssFile.sfcPath?.add(sfcPath);
    matchedMark.add(key);
    cb(cssFile);
    if (cssFile.importer.size > 0) {
      cssFile.importer.forEach((value) => {
        getCSSFileRecursion(lang, value, cssFiles, cb, sfcPath, matchedMark);
      });
    }
  } else {
    throw new Error("path");
  }
};
var getVBindVariableListByPath = (descriptor, id, cssFiles, server, alias) => {
  const vbindVariable = /* @__PURE__ */ new Set();
  const injectCSSContent = /* @__PURE__ */ new Set();
  for (let i = 0; i < descriptor.styles.length; i++) {
    const content = descriptor.styles[i].content;
    const lang = descriptor.styles[i].lang === SUPPORT_FILE.STYLUS ? SUPPORT_FILE.STYL : descriptor.styles[i].lang;
    const idDirParse = parse(id);
    const parseImporterRes = parseImports(content);
    parseImporterRes.imports.forEach((res) => {
      const importerPath = handleAlias(res.path, alias, idDirParse.dir);
      try {
        getCSSFileRecursion(lang, importerPath, cssFiles, (res2) => {
          if (res2.vBindCode) {
            !server && injectCSSContent.add({ content: res2.content, lang: res2.lang, styleTagIndex: i });
            res2.vBindCode.forEach((vb) => {
              vbindVariable.add(vb);
            });
          }
        }, id);
      } catch (e) {
        if (e.message === "path") {
          const doc = "https://github.com/baiwusanyu-c/unplugin-vue-cssvars/pull/29";
          throw new Error(`Unable to resolve file under path '${res.path}', see: ${doc}`);
        } else {
          throw new Error(e.message);
        }
      }
    });
  }
  return {
    vbindVariableListByPath: setTArray(vbindVariable),
    injectCSSContent
  };
};
function handleAlias(path, alias, idDirPath) {
  let importerPath = "";
  if (!alias && !idDirPath)
    return path;
  if (alias) {
    for (const aliasKey in alias) {
      if (alias[aliasKey] && path.startsWith(aliasKey)) {
        importerPath = path.replace(aliasKey, alias[aliasKey]);
        break;
      }
    }
    if (importerPath)
      return transformSymbol(importerPath);
    importerPath = idDirPath ? resolve(idDirPath, path) : path;
  } else {
    idDirPath && (importerPath = resolve(idDirPath, path));
  }
  return transformSymbol(importerPath);
}

// ../packages/core/runtime/pre-process-css.ts
function preProcessCSS(options, alias, filesPath) {
  const { rootDir, includeCompile } = options;
  const files = filesPath || getAllCSSFilePath(includeCompile, rootDir);
  return createCSSFileModuleMap(files, rootDir, alias);
}
function getAllCSSFilePath(includeCompile, rootDir) {
  return fg.sync(includeCompile, {
    ignore: FG_IGNORE_LIST,
    cwd: rootDir
  });
}
function createCSSFileModuleMap(files, rootDir, alias) {
  const cssFiles = /* @__PURE__ */ new Map();
  for (const file of files) {
    let absoluteFilePath = resolve2(parse2(file).dir, parse2(file).base);
    absoluteFilePath = transformSymbol2(absoluteFilePath);
    if (!cssFiles.get(absoluteFilePath)) {
      cssFiles.set(absoluteFilePath, {
        importer: /* @__PURE__ */ new Set(),
        vBindCode: [],
        content: "",
        lang: "css"
      });
    }
  }
  for (const file of files) {
    const fileDirParse = parse2(file);
    const fileSuffix = fileDirParse.ext;
    const code = fs.readFileSync(transformSymbol2(resolve2(rootDir, file)), { encoding: "utf-8" });
    const { imports } = parseImports(code, [transformQuotes]);
    const absoluteFilePath = transformSymbol2(resolve2(fileDirParse.dir, fileDirParse.base));
    const cssF = cssFiles.get(absoluteFilePath);
    imports.forEach((value) => {
      const importerPath = handleAlias(value.path.replace(/^"|"$/g, ""), alias, fileDirParse.dir);
      let importerVal = completeSuffix2(importerPath, SUPPORT_FILE2.CSS);
      if (fileSuffix !== `.${SUPPORT_FILE2.CSS}`) {
        const importerValBySuffix = completeSuffix2(
          importerPath,
          fileSuffix.split(".")[1],
          true
        );
        if (cssFiles.get(importerValBySuffix))
          importerVal = importerValBySuffix;
      }
      cssF.importer.add(importerVal);
    });
    cssF.vBindCode = parseVBindM(code);
    cssF.content = code;
    cssF.lang = fileSuffix.replaceAll(".", "");
    cssFiles.set(absoluteFilePath, cssF);
  }
  return cssFiles;
}

// ../packages/core/option/index.ts
import { resolve as resolve3 } from "path";
import {
  DEFAULT_EXCLUDE_REG,
  DEFAULT_INCLUDE_REG,
  SUPPORT_FILE_LIST,
  extend as extend2
} from "../utils/index.js";
var defaultOption = {
  rootDir: resolve3(),
  include: DEFAULT_INCLUDE_REG,
  exclude: DEFAULT_EXCLUDE_REG,
  revoke: true,
  includeCompile: SUPPORT_FILE_LIST
};
function initOption(option) {
  option = extend2(defaultOption, option);
  return option;
}

// ../packages/core/inject/inject-css.ts
import hash from "hash-sum";

// ../packages/core/transform/transform-inject-css.ts
import MagicString from "magic-string";
function transformInjectCSS(code, importer2) {
  const mgc = new MagicString(code);
  importer2.forEach((imp) => {
    mgc.overwrite(imp.start, imp.end, "");
  });
  return mgc.toString().replaceAll("@import ;", "").replaceAll("v-bind-m", "v-bind");
}

// ../packages/core/inject/inject-css.ts
function injectCssOnServer(code, vbindVariableList, isHmring) {
  vbindVariableList && vbindVariableList.forEach((vbVar) => {
    if (!vbVar.hash && isHmring)
      vbVar.hash = hash(vbVar.value + vbVar.has);
    code = code.replaceAll(`v-bind-m(${vbVar.value})`, `var(--${vbVar.hash})`);
  });
  return code;
}
function injectCssOnBuild(code, injectCSSContent, descriptor) {
  const cssContent = [...injectCSSContent];
  let resCode = "";
  descriptor.styles && descriptor.styles.forEach((value, index) => {
    let injectCssCode = "";
    cssContent.forEach((value2) => {
      if (value2.styleTagIndex === index)
        injectCssCode = `${injectCssCode}
${transformInjectCSS(value2.content, parseImports(value2.content).imports)}`;
    });
    const lang = value.lang || "css";
    const scoped = value.scoped ? "scoped" : "";
    resCode = `<style lang="${lang}" ${scoped}> ${injectCssCode}
${transformInjectCSS(value.content, parseImports(value.content).imports)} </style>`;
  });
  code = removeStyleTagsAndContent(code);
  return `${code}
 ${resCode}`;
}
function removeStyleTagsAndContent(html) {
  const newHtml = html.replace(/<style\b[^>]*>[\s\S]*?<\/style>/gi, "");
  return newHtml.replace(/<style\b[^>]*>/gi, "").replace(/<\/style>/gi, "");
}

// ../packages/core/inject/inject-cssvars.ts
import hash2 from "hash-sum";
var importer = 'import { useCssVars as _useCssVars } from "vue"\n';
var injectCSSVars = (code, vbindVariableList, isScriptSetup) => {
  if (!vbindVariableList || vbindVariableList.length === 0)
    return { code, vbindVariableList };
  return injectCSSVarsOnServer(code, vbindVariableList, isScriptSetup);
};
function injectCSSVarsOnServer(code, vbindVariableList, isScriptSetup) {
  let resCode = "";
  const hasUseCssVars = code.includes("useCssVars");
  const useCssVars = createUseCssVarsCode(code, vbindVariableList, hasUseCssVars, isScriptSetup);
  if (isScriptSetup) {
    if (!hasUseCssVars) {
      resCode = code.replaceAll(
        "setup(__props, { expose }) {",
        `setup(__props, { expose }) {${useCssVars}`
      );
      resCode = `${importer}${resCode}`;
    } else {
      resCode = useCssVars;
    }
  } else {
    if (!hasUseCssVars) {
      resCode = code.replaceAll("const _sfc_main", "const __default__");
      resCode = resCode.replaceAll(
        "function _sfc_render",
        `${useCssVars}

        const __setup__ = __default__.setup
        __default__.setup = __setup__
          ? (props, ctx) => { __injectCSSVars__(); return __setup__(props, ctx) }
            : __injectCSSVars__
            const _sfc_main = __default__
            function _sfc_render`
      );
      resCode = `${importer}${resCode}`;
    } else {
      resCode = useCssVars;
    }
  }
  return { code: resCode, vbindVariableList };
}
function createUseCssVarsCode(code, vbindVariableList, isHas, isScriptSetup) {
  let cssvarsObjectCode = "";
  vbindVariableList.forEach((vbVar) => {
    const hashVal = vbVar.hash || hash2(vbVar.value + vbVar.has);
    vbVar.hash = hashVal;
    let varStr = "";
    if (!isScriptSetup) {
      varStr = `_ctx.${vbVar.value}`;
    } else {
      varStr = vbVar.has ? vbVar.value : `_ctx.${vbVar.value}`;
      varStr = vbVar.isRef ? `${vbVar.value}.value` : varStr;
    }
    cssvarsObjectCode = `"${hashVal}": ${varStr},
  ${cssvarsObjectCode}`;
  });
  let resCode = "";
  if (isHas) {
    resCode = code.includes("_useCssVars((_ctx") ? code.replaceAll(
      "_useCssVars((_ctx) => ({",
      `_useCssVars((_ctx) => ({
  ${cssvarsObjectCode}`
    ) : code.replaceAll(
      "_useCssVars(_ctx => ({",
      `_useCssVars((_ctx) => ({
  ${cssvarsObjectCode}`
    );
  } else {
    resCode = `
     _useCssVars((_ctx) => ({
      ${cssvarsObjectCode}
    }));`;
    if (!isScriptSetup) {
      resCode = `
      const __injectCSSVars__ = () => {

        ${resCode}

      };`;
    }
  }
  return resCode;
}

// ../packages/core/hmr/hmr.ts
import { setTArray as setTArray2 } from "../utils/index.js";
function viteHMR(CSSFileModuleMap, userOptions, file, server) {
  const sfcModulesPathList = CSSFileModuleMap.get(file);
  triggerSFCUpdate(CSSFileModuleMap, userOptions, sfcModulesPathList, file, server);
}
function updatedCSSModules(CSSFileModuleMap, userOptions, file) {
  const updatedCSSMS = preProcessCSS(userOptions, userOptions.alias, [file]).get(file);
  CSSFileModuleMap.set(file, updatedCSSMS);
}
function triggerSFCUpdate(CSSFileModuleMap, userOptions, sfcModulesPathList, file, server) {
  if (sfcModulesPathList && sfcModulesPathList.sfcPath) {
    const ls = setTArray2(sfcModulesPathList.sfcPath);
    ls.forEach((sfcp) => {
      const modules = server.moduleGraph.fileToModulesMap.get(sfcp) || /* @__PURE__ */ new Set();
      updatedCSSModules(CSSFileModuleMap, userOptions, file);
      const modulesList = setTArray2(modules);
      for (let i = 0; i < modulesList.length; i++) {
        if (modulesList[i].id && modulesList[i].id.endsWith(".vue"))
          server.reloadModule(modulesList[i]);
      }
    });
  }
}

// ../packages/core/index.ts
var unplugin = createUnplugin(
  (options = {}) => {
    const userOptions = initOption(options);
    const filter = createFilter(
      userOptions.include,
      userOptions.exclude
    );
    const CSSFileModuleMap = preProcessCSS(userOptions, userOptions.alias);
    const vbindVariableList = /* @__PURE__ */ new Map();
    let isScriptSetup = false;
    if (userOptions.server === void 0) {
      console.warn(chalk.yellowBright.bold(`[${NAME}] The server of option is not set, you need to specify whether you are using the development server or building the project`));
      console.warn(chalk.yellowBright.bold(`[${NAME}] See: https://github.com/baiwusanyu-c/unplugin-vue-cssvars/blob/master/README.md#option`));
    }
    let isServer = !!userOptions.server;
    let isHmring = false;
    return [
      {
        name: NAME,
        enforce: "pre",
        transformInclude(id) {
          return filter(id);
        },
        async transform(code, id) {
          try {
            if (id.endsWith(".vue")) {
              const { descriptor } = parse3(code);
              const lang = descriptor?.script?.lang ?? "js";
              if (JSX_TSX_REG.test(`.${lang}`))
                return code;
              isScriptSetup = !!descriptor.scriptSetup;
              const {
                vbindVariableListByPath,
                injectCSSContent
              } = getVBindVariableListByPath(descriptor, id, CSSFileModuleMap, isServer, userOptions.alias);
              const variableName = getVariable(descriptor);
              vbindVariableList.set(id, matchVariable(vbindVariableListByPath, variableName));
              if (!isServer)
                code = injectCssOnBuild(code, injectCSSContent, descriptor);
            }
            return code;
          } catch (err) {
            this.error(`[${NAME}] ${err}`);
          }
        },
        vite: {
          // Vite plugin
          configResolved(config) {
            if (userOptions.server !== void 0)
              isServer = userOptions.server;
            else
              isServer = config.command === "serve";
          },
          handleHotUpdate(hmr) {
            if (SUPPORT_FILE_REG.test(hmr.file)) {
              isHmring = true;
              viteHMR(
                CSSFileModuleMap,
                userOptions,
                hmr.file,
                hmr.server
              );
            }
          }
        }
      },
      {
        name: `${NAME}:inject`,
        enforce: "post",
        async transform(code, id) {
          try {
            if (isServer) {
              if (id.endsWith(".vue")) {
                const injectRes = injectCSSVars(code, vbindVariableList.get(id), isScriptSetup);
                code = injectRes.code;
                injectRes.vbindVariableList && vbindVariableList.set(id, injectRes.vbindVariableList);
                isHmring = false;
              }
              if (id.includes("type=style"))
                code = injectCssOnServer(code, vbindVariableList.get(id.split("?vue")[0]), isHmring);
            }
            return code;
          } catch (err) {
            this.error(`[${NAME}] ${err}`);
          }
        }
      }
    ];
  }
);
var viteVueCSSVars = unplugin.vite;
var rollupVueCSSVars = unplugin.rollup;
var webpackVueCSSVars = unplugin.webpack;
var esbuildVueCSSVars = unplugin.esbuild;
export {
  esbuildVueCSSVars,
  rollupVueCSSVars,
  viteVueCSSVars,
  webpackVueCSSVars
};
